package pps.core.system.service;

import java.util.*;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import pps.cloud.baseframe.service.FormStateFreeConfCloudService;
import pps.cloud.baseframe.service.data.CreateFormStateFreeConfInput;
import pps.cloud.baseframe.service.data.UpdateFormStateFreeConfInput;
import pps.cloud.primbpm.service.BpsWorkFlowProcessService;
import pps.cloud.primbpm.service.data.*;
import pps.core.common.session.PpsUserSession;
import pps.core.system.constant.OperationMaintenanceManagementConstants;
import pps.core.system.mapper.*;
import pps.core.system.entity.*;
import pps.core.system.service.data.operation_maintenance_adjustment_function_request.ApprovalPassPpsInput;
import pps.core.system.service.data.operation_maintenance_adjustment_function_request.ApprovalRejectPpsInput;
import pps.core.system.service.data.operation_maintenance_adjustment_function_request.QueryOperationMaintenanceAdjustmentFunctionRequestViewOutput;
import pps.core.system.service.data.operation_maintenance_adjustment_function_request.TerminateWorkItemPpsInput;
import pps.core.system.service.data.operation_maintenance_data_modification_application.*;
import pps.core.system.utils.OperationManagementWorkflowBehaviour;
import xstartup.annotation.XService;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;
import xstartup.feature.api.annotation.XApiAnonymous;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.data.XServiceResult;
import xstartup.data.XPageResult;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.error.XError;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

@XText("(运维管理)数据修改申请记录表")
@XService
public class OperationMaintenanceDataModificationApplicationService {

    private final String processCode = "DATA_MODIFICATION_APPLICATION";

    @XText("新增数据修改申请")
    @XApiPost
    public XSingleResult createOperationMaintenanceDataModificationApplication(XContext context, CreateOperationMaintenanceDataModificationApplicationInput input) {
        OperationMaintenanceDataModificationApplicationMapper mapper = context.getBean(OperationMaintenanceDataModificationApplicationMapper.class);
        OperationMaintenanceDataModificationApplicationEnt entity = new OperationMaintenanceDataModificationApplicationEnt();
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        String uuid = UUID.randomUUID().toString().trim().replaceAll("-", "");
        XCopyUtils.copyObject(input, entity);
        entity.setId(uuid);
        //拼接附件filekey
        if (CollectionUtils.isEmpty(input.getFileKeyIds())) {
            entity.setFileKey("");
        } else {
            String fileKey = "";
            for (int i = 0; i < input.getFileKeyIds().size(); i++) {
                if (i == 0) {
                    fileKey = input.getFileKeyIds().get(i);
                } else {
                    fileKey = fileKey + "," + input.getFileKeyIds().get(i);
                }
            }
            entity.setFileKey(fileKey);
        }
        if (input.getDataStatus() != OperationMaintenanceManagementConstants.SAVED) {
            StartProcessInput startProcessInput = new StartProcessInput();
            startProcessInput.setProcessCode(this.processCode);
            startProcessInput.setBsnsDetailUrl(input.getBsnsDetailUrl());
            startProcessInput.setBsnsId(entity.getId());
            //启动相应的工作流转填报
            bpmService.startProcess(context, startProcessInput);
            entity.setAuditstatus(OperationMaintenanceManagementConstants.UNDER_REVIEW);
            entity.setAuditorId(session.getId());
            entity.setAuditorName(session.getUserName());
        } else {
            entity.setAuditstatus(OperationMaintenanceManagementConstants.SAVED);
            entity.setAuditorId(session.getId());
            entity.setAuditorName(session.getUserName());
        }
        //0是删除1是正常存在
        entity.setIsDeleted(OperationMaintenanceManagementConstants.SUBMITTED);
        entity.setmOuId(session.getOuId());
        entity.setmOuName(session.getOuName());
        entity.setCreateById(session.getId());
        entity.setCreateByName(session.getUserName());
        entity.setDataDate(new Date());
        entity.setCreateTime(new Date());
        mapper.insert(entity);
        return XSingleResult.success(entity);
    }


    @XText("根据id查询数据修改申请")
    @XApiPost
    public XSingleResult<QueryOperationMaintenanceDataModificationApplicationViewOutput> getOperationMaintenanceDataModificationApplicationById(XContext context, GetOperationMaintenanceDataModificationApplicationInput input) {
        OperationMaintenanceDataModificationApplicationMapper mapper = context.getBean(OperationMaintenanceDataModificationApplicationMapper.class);
        QueryWrapper<OperationMaintenanceDataModificationApplicationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceDataModificationApplicationEnt::getId, input.getId());
        OperationMaintenanceDataModificationApplicationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XSingleResult.error(context, XError.NotFound);
        }
        QueryOperationMaintenanceDataModificationApplicationViewOutput output = new QueryOperationMaintenanceDataModificationApplicationViewOutput();
        if (!entity.getFileKey().isEmpty()) {
            String[] split = entity.getFileKey().split(",");
            List<String> asList = Arrays.asList(split);
            output.setFileKeyIds(asList);
        }
        XCopyUtils.copyObject(entity, output);
        CheckHasTodoInput checkHasTodoInput = new CheckHasTodoInput();
        checkHasTodoInput.setBsnsId(entity.getId());
        XSingleResult<QueryOperationMaintenanceDataModificationApplicationTypeOutput> result = queryMaintenanceDataTypeOutput(context, checkHasTodoInput);
        int type = result.getResult().getType();
        output.setType(type);
        return XSingleResult.success(output);
    }

    @XText("修改数据修改申请")
    @XApiPost
    public XServiceResult updateOperationMaintenanceDataModificationApplication(XContext context, UpdateOperationMaintenanceDataModificationApplicationInput input) {
        OperationMaintenanceDataModificationApplicationMapper mapper = context.getBean(OperationMaintenanceDataModificationApplicationMapper.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceDataModificationApplicationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceDataModificationApplicationEnt::getId, input.getId());
        OperationMaintenanceDataModificationApplicationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        //拼接附件filekey
        if (CollectionUtils.isEmpty(input.getFileKeyIds())) {
            entity.setFileKey("");
        } else {
            String fileKey = "";
            for (int i = 0; i < input.getFileKeyIds().size(); i++) {
                if (i == 0) {
                    fileKey = input.getFileKeyIds().get(i);
                } else {
                    fileKey = fileKey + "," + input.getFileKeyIds().get(i);
                }
            }
            entity.setFileKey(fileKey);
        }
        entity.setIsDeleted(OperationMaintenanceManagementConstants.SUBMITTED);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        if (null != entity.getAuditstatus()) {
            if (input.getDataStatus() != OperationMaintenanceManagementConstants.SAVED && entity.getAuditstatus() == OperationMaintenanceManagementConstants.REJECT) {
                //启动相应的工作流转填报
                ApprovalPassPpsInput approvalPassPpsInput = new ApprovalPassPpsInput();
                approvalPassPpsInput.setId(input.getId());
                approvedDataModificationApplication(context, approvalPassPpsInput);
                entity.setAuditstatus(OperationMaintenanceManagementConstants.UNDER_REVIEW);
                entity.setAuditorId(session.getId());
                entity.setAuditorName(session.getUserName());
            } else if (input.getDataStatus() != OperationMaintenanceManagementConstants.SAVED) {
                StartProcessInput startProcessInput = new StartProcessInput();
                startProcessInput.setProcessCode(this.processCode);
                startProcessInput.setBsnsDetailUrl(input.getBsnsDetailUrl());
                startProcessInput.setBsnsId(entity.getId());
                //启动相应的工作流转填报
                bpmService.startProcess(context, startProcessInput);
                entity.setAuditstatus(OperationMaintenanceManagementConstants.UNDER_REVIEW);
                entity.setAuditorId(session.getId());
                entity.setAuditorName(session.getUserName());
            }
        }
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("删除数据修改申请")
    @XApiPost
    public XServiceResult deleteOperationMaintenanceDataModificationApplication(XContext context, UpdateOperationMaintenanceDataModificationApplicationInput input) {
        OperationMaintenanceDataModificationApplicationMapper mapper = context.getBean(OperationMaintenanceDataModificationApplicationMapper.class);
        QueryWrapper<OperationMaintenanceDataModificationApplicationEnt> queryWrapper = new QueryWrapper<>();
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        queryWrapper.lambda().eq(OperationMaintenanceDataModificationApplicationEnt::getId, input.getId());
        OperationMaintenanceDataModificationApplicationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        PpsUserSession session = context.getSession(PpsUserSession.class);
        XCopyUtils.copyObject(input, entity);
        entity.setIsDeleted(OperationMaintenanceManagementConstants.SAVED);
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        TerminateWorkItemInput processInput = new TerminateWorkItemInput();
        processInput.setProcessCode(this.processCode);
        processInput.setBsnsId(entity.getId());
        processInput.setRemark("数据修改申请数据源被删除");
        //启动相应的工作流转填报
        bpmService.approvalStop(context, processInput);
        entity.setAuditstatus(OperationMaintenanceManagementConstants.TERMINATION);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("查询数据修改申请")
    @XApiPost
    public XPageResult<QueryOperationMaintenanceDataModificationApplicationViewOutput> queryOperationMaintenanceDataModificationApplicationView(XContext context, QueryOperationMaintenanceDataModificationApplicationViewInput input) {
        OperationMaintenanceDataModificationApplicationViewMapper mapper = context.getBean(OperationMaintenanceDataModificationApplicationViewMapper.class);
        OperationMaintenanceDataModificationApplicationView record = new OperationMaintenanceDataModificationApplicationView();
        List<QueryOperationMaintenanceDataModificationApplicationViewOutput> arrayList = new ArrayList();
        XCopyUtils.copyObject(input, record);
        record.setIsDeleted(OperationMaintenanceManagementConstants.SUBMITTED);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<OperationMaintenanceDataModificationApplicationView> list = mapper.selectList(record);
        PageInfo<OperationMaintenanceDataModificationApplicationView> pageInfo = new PageInfo<>(list);
        List<QueryOperationMaintenanceDataModificationApplicationViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QueryOperationMaintenanceDataModificationApplicationViewOutput.class);
        if (outputs.size() > 0) {
            for (int i = 0; i < outputs.size(); i++) {
                QueryOperationMaintenanceDataModificationApplicationViewOutput viewOutput = outputs.get(i);
                if (!viewOutput.getFileKey().isEmpty()) {
                    String[] split = viewOutput.getFileKey().split(",");
                    List<String> asList = Arrays.asList(split);
                    viewOutput.setFileKeyIds(asList);
                }
                CheckHasTodoInput checkHasTodoInput = new CheckHasTodoInput();
                checkHasTodoInput.setBsnsId(viewOutput.getId());
                XSingleResult<QueryOperationMaintenanceDataModificationApplicationTypeOutput> result = queryMaintenanceDataTypeOutput(context, checkHasTodoInput);
                int type = result.getResult().getType();
                viewOutput.setType(type);
                arrayList.add(viewOutput);
            }
        }
        PageInfo<QueryOperationMaintenanceDataModificationApplicationViewOutput> outputPageInfo = new PageInfo<>(arrayList);
        List<QueryOperationMaintenanceDataModificationApplicationViewOutput> arrayListOutputs = XCopyUtils.copyNewList(outputPageInfo.getList(), QueryOperationMaintenanceDataModificationApplicationViewOutput.class);
        return XPageResult.success(arrayListOutputs, input, pageInfo.getTotal());
    }


    @XText("审核通过数据修改申请")
    @XApiPost
    public XServiceResult approvedDataModificationApplication(XContext context, ApprovalPassPpsInput input) {
        OperationMaintenanceDataModificationApplicationMapper mapper = context.getBean(OperationMaintenanceDataModificationApplicationMapper.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceDataModificationApplicationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceDataModificationApplicationEnt::getId, input.getId());
        OperationMaintenanceDataModificationApplicationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        //调用工作流通过接口
        FinishWorkItemDirectlyInput finishWorkItemDirectlyInput = new FinishWorkItemDirectlyInput();
        //流程编号
        finishWorkItemDirectlyInput.setWorkItemId(input.getWorkItemId());
        finishWorkItemDirectlyInput.setProcessCode(processCode);
        finishWorkItemDirectlyInput.setRemark(input.getRemark());
        finishWorkItemDirectlyInput.setBsnsId(entity.getId());
        XServiceResult xServiceResult = bpmService.approvalPass(context, finishWorkItemDirectlyInput);
        xServiceResult.throwIfFail();
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setAuditorId(session.getId());
        entity.setAuditorName(session.getUserName());
        GetProcessInstStateInput getProcessInstStateInput = new GetProcessInstStateInput();
        getProcessInstStateInput.setBsnsId(input.getId());
        getProcessInstStateInput.setProcessCode(processCode);
        XSingleResult<GetProcessInstStateOutput> getProcessInstStateOutputXSingleResult = bpmService.queryProcessStatus(context, getProcessInstStateInput);
        if (getProcessInstStateOutputXSingleResult.getResult().getState() == 2) {
            entity.setAuditstatus(OperationMaintenanceManagementConstants.UNDER_REVIEW);
        } else if (getProcessInstStateOutputXSingleResult.getResult().getState() == 7) {
            entity.setAuditstatus(OperationMaintenanceManagementConstants.PASSED);
        }
        mapper.updateById(entity);
        return XServiceResult.OK;
    }


    @XText("审核驳回数据修改申请")
    @XApiPost
    public XServiceResult reviewRejectDataModificationApplication(XContext context, ApprovalRejectPpsInput input) {
        OperationMaintenanceDataModificationApplicationMapper mapper = context.getBean(OperationMaintenanceDataModificationApplicationMapper.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceDataModificationApplicationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceDataModificationApplicationEnt::getId, input.getId());
        OperationMaintenanceDataModificationApplicationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        //调用工作流通过接口
        pps.cloud.primbpm.service.data.ApprovalRejectInput approvalRejectInput = new pps.cloud.primbpm.service.data.ApprovalRejectInput();
        //获取参数
        approvalRejectInput.setBackReason(input.getBackReason());
        approvalRejectInput.setBsnsId(entity.getId());
        approvalRejectInput.setProcessCode(processCode);
        XServiceResult xServiceResult = bpmService.approvalReject(context, approvalRejectInput);
        xServiceResult.throwIfFail();
        //填充用户信息
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setAuditorId(session.getId());
        entity.setAuditorName(session.getUserName());
        entity.setAuditstatus(OperationMaintenanceManagementConstants.REJECT);
        entity.setDataStatus(OperationMaintenanceManagementConstants.SUBMITTED);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }

    @XText("终止数据修改申请")
    @XApiPost
    public XServiceResult terminateDataModificationApplication(XContext context, TerminateWorkItemPpsInput input) {
        OperationMaintenanceDataModificationApplicationMapper mapper = context.getBean(OperationMaintenanceDataModificationApplicationMapper.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceDataModificationApplicationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceDataModificationApplicationEnt::getId, input.getId());
        OperationMaintenanceDataModificationApplicationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        pps.cloud.primbpm.service.data.TerminateWorkItemInput workItemInput = new pps.cloud.primbpm.service.data.TerminateWorkItemInput();
        workItemInput.setRemark(input.getRemark());
        workItemInput.setBsnsId(entity.getId());
        workItemInput.setProcessCode(processCode);
        XServiceResult xServiceResult = bpmService.approvalStop(context, workItemInput);
        xServiceResult.throwIfFail();
        //填充用户信息
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setAuditorId(session.getId());
        entity.setAuditorName(session.getUserName());
        entity.setAuditstatus(OperationMaintenanceManagementConstants.TERMINATION);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }


    @XText("接受需求数据修改申请")
    @XApiPost
    public XServiceResult acceptDemandDataModificationApplication(XContext context, TerminateWorkItemPpsInput input) {
        OperationMaintenanceDataModificationApplicationMapper mapper = context.getBean(OperationMaintenanceDataModificationApplicationMapper.class);
        BpsWorkFlowProcessService bpmService = context.getBean(BpsWorkFlowProcessService.class);
        FormStateFreeConfCloudService fscService= context.getBean(FormStateFreeConfCloudService.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryWrapper<OperationMaintenanceDataModificationApplicationEnt> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(OperationMaintenanceDataModificationApplicationEnt::getId, input.getId());
        OperationMaintenanceDataModificationApplicationEnt entity = mapper.selectOne(queryWrapper);
        if (entity == null) {
            return XServiceResult.error(context, XError.NotFound);
        }
        XCopyUtils.copyObject(input, entity);
        pps.cloud.primbpm.service.data.TerminateWorkItemInput workItemInput = new pps.cloud.primbpm.service.data.TerminateWorkItemInput();
        workItemInput.setRemark(input.getRemark());
        workItemInput.setBsnsId(entity.getId());
        workItemInput.setProcessCode(processCode);
        XServiceResult xServiceResult = bpmService.approvalAcceptDemand(context, workItemInput);
        xServiceResult.throwIfFail();
        //填充用户信息
        entity.setModifyById(session.getId());
        entity.setModifyByName(session.getUserName());
        entity.setModifyTime(new Date());
        entity.setAuditorId(session.getId());
        entity.setAuditorName(session.getUserName());
        entity.setAuditstatus(OperationMaintenanceManagementConstants.ACCEPTDEMAND);
        //调用数据释放徐成栋的修改接口
        UpdateFormStateFreeConfInput freeConfInput = new UpdateFormStateFreeConfInput();
        freeConfInput.setId(entity.getFormTemplatesId());
        fscService.updateFormStateFreeConf(context,freeConfInput);
        mapper.updateById(entity);
        return XServiceResult.OK;
    }



    /*
    *   填报+审核：审核通过+审核驳回（3）
        填报+审核：中止+审核通过+审核驳回（8）
        填报+审核：接受需求+中止+审核通过+审核驳回（9）
        填报+审核：接受需求+中止+审核驳回（10）

        填报：提交（0）

        审核：审核通过+审核驳回（1）
        审核：中止+审核通过+审核驳回（5）
        审核：接受需求+中止+审核通过+审核驳回（6）
        审核：接受需求+中止+审核驳回（7）

        没有角色：（2）
    *
    * */

    @XText("查询当前登陆人角色状态--前端暂不调用")
    @XApiPost
    public XSingleResult<QueryOperationMaintenanceDataModificationApplicationTypeOutput> queryMaintenanceDataTypeOutput(XContext context, CheckHasTodoInput input) {
        //查询流程下的所有角色
        BpsWorkFlowProcessService bean = context.getBean(BpsWorkFlowProcessService.class);
        // 获取用户角色信息
        //SystemUserRoleRelService service = context.getBean(SystemUserRoleRelService.class);
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryOperationMaintenanceDataModificationApplicationTypeOutput output = new QueryOperationMaintenanceDataModificationApplicationTypeOutput();
        QueryProcessRolesInput queryProcessRolesInput = new QueryProcessRolesInput();
        queryProcessRolesInput.setProcessCode(processCode);
        //获取流程第一个节点的参与者信息（角色信息）  processCode    session.getId() 获取当前登陆人的角色判断 是否一样
        OperationManagementWorkflowBehaviour behaviour = new OperationManagementWorkflowBehaviour();
        //判断当前用户是否为当前流程的 第一个节点角色
        boolean flag = behaviour.checkInRoleOrUserId(context, processCode, session.getId());
        //判断当前用户不是当前流程的任何角色
        //boolean type = behaviour.checkInRoleOrUserIds(context, processCode, session.getId());

        //当前用户同时拥有 提报跟审批角色 判断当前用户是否有代办
        XSingleResult<CheckHasTodoOutput> checkHasTodoOutputXSingleResult = bean.checkHasTodo(context, input);
        int roles = queryUsersRoles(context);
        if (checkHasTodoOutputXSingleResult.getResult().getHasTodo() && flag) {
            if (roles == 4) {
                output.setType(3);
            } else if (roles == 5) {
                output.setType(8);
            } else if (roles == 6) {
                output.setType(9);
            } else if (roles == 7) {
                output.setType(10);
            }
        } else if (flag) {
            //当前用户拥有 提报角色
            output.setType(0);
        } else if (checkHasTodoOutputXSingleResult.getResult().getHasTodo()) {
            //审核角色
            if (roles == 4) {
                output.setType(1);
            } else if (roles == 5) {
                output.setType(5);
            } else if (roles == 6) {
                output.setType(6);
            } else if (roles == 7) {
                output.setType(7);
            }
        } else {
            //没有角色
            output.setType(2);
        }
        return XSingleResult.success(output);
    }

    @XText("查询当前登陆人角色状态")
    @XApiPost
    public XSingleResult<QueryOperationMaintenanceDataModificationApplicationTypeOutput> queryMaintenanceDataTypeOutputNotHave(XContext context) {
        //获取用户信息
        PpsUserSession session = context.getSession(PpsUserSession.class);
        QueryOperationMaintenanceDataModificationApplicationTypeOutput output = new QueryOperationMaintenanceDataModificationApplicationTypeOutput();
        //获取流程第一个节点的参与者信息（角色信息）  processCode    session.getId() 获取当前登陆人的角色判断 是否一样
        OperationManagementWorkflowBehaviour behaviour = new OperationManagementWorkflowBehaviour();
        //判断当前用户是否为当前流程的 第一个节点角色
        boolean flag = behaviour.checkInRoleOrUserId(context, processCode, session.getId());
        if (flag) {
            //当前用户拥有 提报角色
            output.setType(0);
        } else {
            //没有角色
            output.setType(2);
        }
        return XSingleResult.success(output);
    }

    @XText("查询当前登陆人角色状态")
    public int  queryUsersRoles(XContext context) {
        PpsUserSession session = context.getSession(PpsUserSession.class);
        SysRoleViewMapper sysRoleViewMapper = context.getBean(SysRoleViewMapper.class);
        List<SysRoleExtendView> roleList = sysRoleViewMapper.getRoleListUserId(session.getId());
        if (!CollectionUtils.isEmpty(roleList)){
            for (int i = 0; i < roleList.size(); i++) {
                SysRoleExtendView view = roleList.get(i);
                if (view.getRoleName().equals("在线运维管理_三线运维人员")){
                    return OperationMaintenanceManagementConstants.DISCONTINUE;
                }else if (view.getRoleName().equals("PPS运维组")){
                    return OperationMaintenanceManagementConstants.SUSPENSIONACCEPTANCE;
                }else if (view.getRoleName().equals("PPS技术组")){
                    return OperationMaintenanceManagementConstants.SUSPENDACCEPTSUBMISSIONS;
                }
            }
        }
        return OperationMaintenanceManagementConstants.NOOPERATION;
    }

}
