package pps.core.system.service;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import org.apache.logging.log4j.util.Strings;
import org.springframework.util.CollectionUtils;
import pps.cloud.base.service.BaseBaseStationService;
import pps.cloud.base.service.data.BaseStationIdInput;
import pps.cloud.base.service.data.QueryBaseStationOutput;
import pps.cloud.framedata.service.CommonDataService;
import pps.cloud.framedata.service.data.common_data.SelectCommonInput;
import pps.cloud.framedata.service.data.common_data.SelectCommonOutput;
import pps.cloud.system.service.SysInformDayDesChangeCloudService;
import pps.cloud.system.service.SystemDictionaryService;
import pps.cloud.system.service.data.*;
import pps.core.common.config.MqConfig;
import pps.core.common.mq.SysInformSendMqMessage;
import pps.core.common.session.PpsUserSession;
import pps.core.common.utils.TraceHelper;
import pps.core.system.entity.*;
import pps.core.system.mapper.*;
import pps.core.system.service.data.sys_inform.SendSysInformInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.mq.XMqFactory;
import xstartup.base.mq.XMqProduceQueue;
import xstartup.base.mq.XMqProducer;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XJsonUtils;
import xstartup.core.base.helper.XThreadHelper;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.helper.XTransactionHelper;

import java.net.CacheRequest;
import java.util.*;

import static javax.swing.UIManager.get;

@XService
public class SysInformDayDesChangeCloudServiceImpl implements SysInformDayDesChangeCloudService {

    private String SEND_OU_ID ;
    private String SEND_OU_NAME;
    private String SEND_USER_ID;
    private String SEND_USER_NAME;
    private String NG_DAY_DES_INFORM_ROLE = "GAS_NG_DAY_DES_INFORM_ROLE";
    private String LNG_DAY_DES_INFORM_ROLE = "LNG_DAY_DES_INFORM_ROLE";

    private void initSendUser(XContext context,String type){
        if("lng".equals(type)){
            PpsUserSession session = context.getSession(PpsUserSession.class);
            SEND_OU_ID = session.getOuId();
            SEND_OU_NAME = session.getOuName();
            SEND_USER_ID = session.getId();
            SEND_USER_NAME = session.getUserName();
        }else{
           try {
               String sql = "select so.id,so.ou_name from pps_base_info.sys_organization_rel sor " +
                       "left join pps_base_info.sys_organization so on sor.ou_id = so.id " +
                       "where sor.levels = 1 and sor.end_time >sysdate";
               CommonDataService service = context.getBean(CommonDataService.class);
               SelectCommonInput input = new SelectCommonInput();
               input.setCmd(sql);
               XSingleResult<SelectCommonOutput> result = service.selectCommon(context, input);
               if(result.isSuccess()){
                   List<LinkedHashMap<String, Object>> list = result.getResult().getResultList();
                   if(list.size()>0){
                       SEND_OU_ID = String.valueOf(list.get(0).get("id"));
                       SEND_OU_NAME = String.valueOf(list.get(0).get("ou_name"));
                   }
               }
           }catch (Exception e){
               e.printStackTrace();
           }
           if(SEND_OU_ID == null){
               SEND_OU_ID = "F6AC16F3-C184-4836-BE87-65AA5F1695B8";
               SEND_OU_NAME = "国家石油天然气管网集团有限公司";
           }
            SEND_USER_ID = "gdq_async_send";
            SEND_USER_NAME = "过渡期推送";
        }
    }

    private Date date;

    @XText("管输日指定变更发送消息")
    public XServiceResult sendInformForNgDayDes(XContext context, SendInformForDayDesInput input) {
        TraceHelper.buildTraceBuilder(context);
        try {
            TraceHelper.appendTraceLog(context, "sendInformForNgDayDes start：params:",  XJsonUtils.toJson(input));
            this.initSendUser(context,"ng");
            this.sendInforms(context,input,NG_DAY_DES_INFORM_ROLE);
            TraceHelper.appendTraceLog(context, "sendInformForNgDayDes end");
            return XServiceResult.OK;
        }catch (Exception e){
            TraceHelper.appendTraceLog(context, e);
            return XServiceResult.error(e);
        }finally {
            TraceLogger.INSTANCE.add(context);
        }

    }

    @Override
    @XText("lng装车日指定下发发送消息")
    public XServiceResult sendInformForLngDayDesRelease(XContext context, SendInformForDayDesInput input) {
        TraceHelper.buildTraceBuilder(context);
        try {
            TraceHelper.appendTraceLog(context, "sendInformForNgDayDes start：params:",  XJsonUtils.toJson(input));
            this.initSendUser(context,"lng");
            this.sendInforms(context,input,LNG_DAY_DES_INFORM_ROLE);
            TraceHelper.appendTraceLog(context, "sendInformForNgDayDes end");
            return XServiceResult.OK;
        }catch (Exception e){
            TraceHelper.appendTraceLog(context, e);
            return XServiceResult.error(e);
        }finally {
            TraceLogger.INSTANCE.add(context);
        }
    }

    @Override
    @XText("lng装车日指定取消/恢复发送消息")
    public XServiceResult sendInformForLngDayDesUpOrDown(XContext context, SendInformForDayDesInput input) {
        TraceHelper.buildTraceBuilder(context);
        try {
            TraceHelper.appendTraceLog(context, "sendInformForLngDayDesUpOrDown start：params:", XJsonUtils.toJson(input));
            this.initSendUser(context,"lng");
            this.sendInforms(context,input,LNG_DAY_DES_INFORM_ROLE);
            TraceHelper.appendTraceLog(context, "sendInformForLngDayDesUpOrDown end");
            return XServiceResult.OK;
        }catch (Exception e){
            TraceHelper.appendTraceLog(context, e);
            return XServiceResult.error(e);
        }finally {
            TraceLogger.INSTANCE.add(context);
        }
    }

    private void sendInforms(XContext context,SendInformForDayDesInput input,String roleAlias){
        if(CollectionUtils.isEmpty(input.getDataList())){
            context.getLogger().error("not find dataList");
            return;
        }
        List<ObjForSendInform> roleList = this.getInformRoleList(context, roleAlias);
        if(roleList.isEmpty()) {
            context.getLogger().error(String.format("not find role config : Dictionary alias %s",roleAlias));
            return;
        }
        date = new Date();
        for(SendInformForDayDesItem item:input.getDataList()){
            XThreadHelper.async(new Runnable() {
                @Override
                public void run() {
                    sendInform(context,item,roleList);
                }
            });
        }
    }


    private void sendInform(XContext context,SendInformForDayDesItem data,List<ObjForSendInform> roleList){
        String sysInformId = UUID.randomUUID().toString();
        List<ObjForSendInform> ouList = this.getOusForSendInform(context, data);
        if(ouList.isEmpty()){
            context.getLogger().error("");
        }

        //消息实体
        SysInformEnt sysInformEnt = new SysInformEnt();
        //消息类型集合
        List<SysInformReceiveTypeEnt> receiveTypeEntList = new ArrayList<>();
        //消息类型详情集合
        List<SysInformReceiveTypeDetailEnt> typeDetailEntList = new ArrayList<>();

        //创建消息
        this.createInform(data,sysInformId,sysInformEnt);
        //创建消息类型
        this.createInformType5(context,ouList,sysInformId,roleList,receiveTypeEntList,typeDetailEntList);
        this.createInformType6(context,ouList,sysInformId,roleList,receiveTypeEntList,typeDetailEntList);
        this.createInformType7(context,ouList,sysInformId,roleList,receiveTypeEntList,typeDetailEntList);

        //保存消息
        XTransactionHelper.begin(context,()->{
            if(receiveTypeEntList.size()>0 && typeDetailEntList.size()>0){

                SysInformMapper informMapper = context.getBean(SysInformMapper.class);
                SysInformReceiveTypeMapper receiveTypeMapper =  context.getBean(SysInformReceiveTypeMapper.class);
                SysInformReceiveTypeDetailMapper receiveTypeDetailMapper = context.getBean(SysInformReceiveTypeDetailMapper.class);

                informMapper.insert(sysInformEnt);

                for(SysInformReceiveTypeEnt item :receiveTypeEntList){
                    receiveTypeMapper.insert(item);
                }

                for(SysInformReceiveTypeDetailEnt item :typeDetailEntList){
                    receiveTypeDetailMapper.insert(item);
                }
            }
            return XServiceResult.OK;
        });
        //调用发送通知
        SendSysInformInput informInput = new SendSysInformInput();
        informInput.setId(sysInformId);
        sendSysInformMq(context,informInput);
    }

    private void createInform(SendInformForDayDesItem data,String sysInformId,SysInformEnt sysInformEnt){
        sysInformEnt.setId(sysInformId);
        sysInformEnt.setTitle(data.getTitle());
        sysInformEnt.setMsg(data.getMsg());
        sysInformEnt.setOuId(SEND_OU_ID);
        sysInformEnt.setOuName(SEND_OU_NAME);
        sysInformEnt.setUserId(SEND_USER_ID);
        sysInformEnt.setUserName(SEND_USER_NAME);
        sysInformEnt.setSendType(0);
        sysInformEnt.setSystemMsg(1);
        sysInformEnt.setMobileMsg(0);
        sysInformEnt.setSendTime(date);
        sysInformEnt.setStatus(0);
        sysInformEnt.setCreateTime(date);
        sysInformEnt.setCreateById(SEND_USER_ID);
        sysInformEnt.setCreateByName(SEND_USER_NAME);
    }

    private XServiceResult sendSysInformMq(XContext context, SendSysInformInput input){
        return XTransactionHelper.begin(context , ()->{
            //判断创建和更新操作

            SysInformMapper mapper = context.getBean(SysInformMapper.class);
            QueryWrapper<SysInformEnt> wrapper = new QueryWrapper<>();
            wrapper.lambda().eq(SysInformEnt::getId , input.getId());
            SysInformEnt ent = mapper.selectOne(wrapper);
            if(ent == null)
                return XServiceResult.error(context, XError.NotFound);
            if(ent.getStatus() >= 1)
                return XServiceResult.error(200, "数据状态不支持发送！");
            SysInformSendMqMessage sysInformSendMqMessage =new SysInformSendMqMessage();
            sysInformSendMqMessage.setId(input.getId());
            sendMqSysInform(context, sysInformSendMqMessage);
            //执行完成后，更新状态
            ent.setStatus(1);
            ent.setModifyTime(date);
            ent.setSendTime(date);
            mapper.updateById(ent);
            return XServiceResult.OK;
        });
    }

    public GetSysOrganizationViewOutput getSysOrganizationByOuLevelAndOuId(XContext context, SysOrganizationView view) {
        SysOrganizationViewMapper viewMapper = context.getBean(SysOrganizationViewMapper.class);
        view.setEndTime(new Date());
        SysOrganizationView sysOrganizationView = viewMapper.getSysOrganizationByOuLevelAndOuId(view);
        return XCopyUtils.copyNewObject(sysOrganizationView, GetSysOrganizationViewOutput.class);
    }

    public void sendMqSysInform(XContext context, SysInformSendMqMessage sysInformSendMqMessage) {
        String schedulTopic = MqConfig.sysInformSendMq.getValue(context);
        XMqProduceQueue queue = new XMqProduceQueue.Builder().queue(schedulTopic).build();
        XMqProducer<SysInformSendMqMessage> producer = XMqFactory.createProducer(context, queue, SysInformSendMqMessage.class);
        producer.publish(context, sysInformSendMqMessage).logIfFail(context);
    }

    /**
     * 添加消息-角色-本级单位
     */
    private void createInformType5(XContext context,List<ObjForSendInform> ouList,String sysInformId,List<ObjForSendInform> roleList,
                                   List<SysInformReceiveTypeEnt> receiveTypeEntList,List<SysInformReceiveTypeDetailEnt> typeDetailEntList){
        if(ouList.isEmpty() || roleList.isEmpty()) return;
        TraceHelper.appendTraceLog(context, "接收类型为角色-提交人本级单位", "开始拼接参数");

        //添加加消息类型
        String typeEntId = UUID.randomUUID().toString();
        SysInformReceiveTypeEnt receiveTypeEnt = new SysInformReceiveTypeEnt();
        receiveTypeEnt.setId(typeEntId);
        receiveTypeEnt.setInformId(sysInformId);
        receiveTypeEnt.setRecieveType(5);
        receiveTypeEnt.setRecieveTypeName("角色-提交人本级单位");
        receiveTypeEntList.add(receiveTypeEnt);

        for(ObjForSendInform ou:ouList){
            for(ObjForSendInform role:roleList){
                //添加消息详情
                SysInformReceiveTypeDetailEnt typeDetailEnt = new SysInformReceiveTypeDetailEnt();
                typeDetailEnt.setId(UUID.randomUUID().toString());
                typeDetailEnt.setInformId(sysInformId);
                typeDetailEnt.setInformRecieveTypeId(typeEntId);
                typeDetailEnt.setOuId(ou.getId());
                typeDetailEnt.setOuName(ou.getName());
                typeDetailEnt.setObjId(role.getId());
                typeDetailEnt.setObjName(role.getName());
                typeDetailEntList.add(typeDetailEnt);
            }
        }
        TraceHelper.appendTraceLog(context, "接收类型为角色-提交人本级单位的接收信息-结束");
    }

    /**
     * 添加消息-角色-提交人二级单位
     */
    private void createInformType6(XContext context,
                                   List<ObjForSendInform> ouList,String sysInformId,List<ObjForSendInform> roleList,
                                   List<SysInformReceiveTypeEnt> receiveTypeEntList,List<SysInformReceiveTypeDetailEnt> typeDetailEntList){
        if(ouList.isEmpty() || roleList.isEmpty()) return;
        TraceHelper.appendTraceLog(context, "接收类型为角色-提交人二级单位", "开始拼接参数");

        //添加加消息类型
        String typeEntId = UUID.randomUUID().toString();
        SysInformReceiveTypeEnt receiveTypeEnt = new SysInformReceiveTypeEnt();
        receiveTypeEnt.setId(typeEntId);
        receiveTypeEnt.setInformId(sysInformId);
        receiveTypeEnt.setRecieveType(6);
        receiveTypeEnt.setRecieveTypeName("角色-提交人二级单位");
        receiveTypeEntList.add(receiveTypeEnt);

        for(ObjForSendInform ou:ouList){
            SysOrganizationView view = new SysOrganizationView();
            view.setOuLevel("ADMINISTRATIVE");
            view.setId(ou.getId());
            GetSysOrganizationViewOutput organization = getSysOrganizationByOuLevelAndOuId(context, view);
            for(ObjForSendInform role:roleList){
                //添加消息详情
                SysInformReceiveTypeDetailEnt typeDetailEnt = new SysInformReceiveTypeDetailEnt();
                typeDetailEnt.setId(UUID.randomUUID().toString());
                typeDetailEnt.setInformId(sysInformId);
                typeDetailEnt.setInformRecieveTypeId(typeEntId);
                typeDetailEnt.setOuId(organization.getId());
                typeDetailEnt.setOuName(organization.getOuName());
                typeDetailEnt.setObjId(role.getId());
                typeDetailEnt.setObjName(role.getName());
                typeDetailEntList.add(typeDetailEnt);
            }
        }
        TraceHelper.appendTraceLog(context, "接收类型为角色-提交人二级单位的接收信息-结束");
    }

    /**
     * 添加消息-角色-提交人所属企业
     */
    private void createInformType7(XContext context,
                                   List<ObjForSendInform> ouList,String sysInformId,List<ObjForSendInform> roleList,
                                   List<SysInformReceiveTypeEnt> receiveTypeEntList,List<SysInformReceiveTypeDetailEnt> typeDetailEntList){
        if(ouList.isEmpty() || roleList.isEmpty()) return;
        TraceHelper.appendTraceLog(context, "接收类型为角色-提交人所属企业", "开始拼接参数");

        //添加加消息类型
        String typeEntId = UUID.randomUUID().toString();
        SysInformReceiveTypeEnt receiveTypeEnt = new SysInformReceiveTypeEnt();
        receiveTypeEnt.setId(typeEntId);
        receiveTypeEnt.setInformId(sysInformId);
        receiveTypeEnt.setRecieveType(7);
        receiveTypeEnt.setRecieveTypeName("角色-提交人所属企业");
        receiveTypeEntList.add(receiveTypeEnt);

        for(ObjForSendInform ou:ouList){
            SysOrganizationView input = new SysOrganizationView();
            input.setOuLevel("REGION_FIRM");
            input.setId(ou.getId());
            GetSysOrganizationViewOutput organization = getSysOrganizationByOuLevelAndOuId(context, input);
            for(ObjForSendInform role:roleList){
                //添加消息详情
                SysInformReceiveTypeDetailEnt typeDetailEnt = new SysInformReceiveTypeDetailEnt();
                typeDetailEnt.setId(UUID.randomUUID().toString());
                typeDetailEnt.setInformId(sysInformId);
                typeDetailEnt.setInformRecieveTypeId(typeEntId);
                typeDetailEnt.setOuId(organization.getId());
                typeDetailEnt.setOuName(organization.getOuName());
                typeDetailEnt.setObjId(role.getId());
                typeDetailEnt.setObjName(role.getName());
                typeDetailEntList.add(typeDetailEnt);
            }
        }

        TraceHelper.appendTraceLog(context, "接收类型为角色-提交人所属企业的接收信息-结束");
    }

    private List<ObjForSendInform> getOusForSendInform(XContext context,SendInformForDayDesItem data){

        List<ObjForSendInform> resultList = new ArrayList<>();

        if(!Strings.isBlank(data.getStationId())){
            ObjForSendInform item = this.getOuByStationId(context, data.getStationId());
            if(item != null) resultList.add(item);
        }else if(!CollectionUtils.isEmpty(data.getStationIdList())){
            Set<String> ouIdSet = new HashSet<>();
            for(String stationId : data.getStationIdList()){
                ObjForSendInform item = this.getOuByStationId(context,stationId);
                if(item != null && !ouIdSet.contains(item.getId())){
                    ouIdSet.add(stationId);
                    resultList.add(item);
                }
            }
        }
        return resultList;
    }

    private ObjForSendInform getOuByStationId(XContext context,String stationId){
        BaseBaseStationService service = context.getBean(BaseBaseStationService.class);
        BaseStationIdInput input = new BaseStationIdInput();
        input.setStationId(stationId);
        XListResult<QueryBaseStationOutput> outputs = service.selectStationOuByStationId(context, input);
        if(outputs.isSuccess() && !CollectionUtils.isEmpty(outputs.getResult())){
            QueryBaseStationOutput item = outputs.getResult().get(0);
            return new ObjForSendInform(item.getOuId(),item.getOuName());
        }
        return null;
    }

    private List<ObjForSendInform> getInformRoleList(XContext context,String alias){
        List<ObjForSendInform> resultList = new ArrayList<>();
        SystemDictionaryService dictService = context.getBean(SystemDictionaryService.class);
        CreateSysDictionaryInput viewInput = new CreateSysDictionaryInput();
        viewInput.setAlias(alias);
        XListResult<QuerySysDictionaryViewOutput> dictionarys = dictService.querySysDictionarysByAlias(context, viewInput);
        if(dictionarys.isSuccess() && !CollectionUtils.isEmpty(dictionarys.getResult())){
            List<QuerySysDictionaryViewOutput> dicList = dictionarys.getResult();
            for(QuerySysDictionaryViewOutput item:dicList){
                resultList.add(new ObjForSendInform(item.getDicKey(),item.getDicName()));
            }
        }
        return resultList;
    }
}
