package pps.core.system.service;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;
import org.springframework.util.CollectionUtils;
import pps.cloud.dispatch.service.DispDutyMatterYqdkzxInfCloudService;
import pps.cloud.dispatch.service.DispOrderInforCloudService;
import pps.cloud.dispatch.service.DispReceiveGroupReceiverCloudService;
import pps.cloud.dispatch.service.data.GetDispDutyMatterYqdkzxInfTypeDetaiCloudOutput;
import pps.cloud.dispatch.service.data.GetDispOrderInfoCloudInput;
import pps.cloud.dispatch.service.data.GetDispOrderInfoCloudOutput;
import pps.core.common.session.PpsUserSession;
import pps.core.system.entity.SysInformEnt;
import pps.core.system.entity.SysInformReceiverEnt;
import pps.core.system.entity.SysInformReceiverView;
import pps.core.system.entity.SysMenuEnt;
import pps.core.system.mapper.SysInformReceiverMapper;
import pps.core.system.mapper.SysInformReceiverViewMapper;
import pps.core.system.service.data.sys_inform_receiver.*;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XCopyUtils;
import xstartup.base.util.XStringUtils;
import xstartup.data.XListResult;
import xstartup.data.XPageResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.error.XError;
import xstartup.feature.api.annotation.XApiGet;
import xstartup.feature.api.annotation.XApiPost;
import xstartup.helper.XTransactionHelper;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@XService
@XText("消息通知-接收方")
public class SysInformReceiverService {

    @XText("查询接收方列表数据")
    @XApiGet
    public XPageResult<QuerySysInformReceiverViewOutput> selectSysInformReceiverPage(XContext context, QuerySysInformReceiverViewInput input){
        SysInformReceiverViewMapper mapper = context.getBean(SysInformReceiverViewMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        if(XStringUtils.isEmpty(input.getInformId()))
            input.setUserId(session.getId());
        SysInformReceiverView record = new SysInformReceiverView();
        XCopyUtils.copyObject(input, record);
        PageHelper.startPage(input.getPage(), input.getLimit());
        List<SysInformReceiverView> list = mapper.selectList(record);
        PageInfo<SysInformReceiverView> pageInfo = new PageInfo<>(list);
        List<QuerySysInformReceiverViewOutput> outputs = XCopyUtils.copyNewList(pageInfo.getList(), QuerySysInformReceiverViewOutput.class);
        return XPageResult.success(outputs, input, pageInfo.getTotal());
    }

    @XText("查看数据")
    @XApiGet
    public XSingleResult<GetSysInformReceiverViewOutput> getSysInformReceiver(XContext context, GetSysInformReceiverViewInput input){
        SysInformReceiverMapper mapper = context.getBean(SysInformReceiverMapper.class);
        QueryWrapper<SysInformReceiverEnt> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(SysInformReceiverEnt::getId , input.getId());
        SysInformReceiverEnt ent = mapper.selectOne(wrapper);
        if(ent == null)
            return XSingleResult.error(context, XError.NotFound);
        //查看数据后，更新数据状态
        if(ent.getStatus() == 0){
            PpsUserSession session = context.getSession(PpsUserSession.class);
            Date date = new Date();
            ent.setStatus(1);
            ent.setModifyTime(date);
            ent.setModifyByName(session.getUserName());
            ent.setModifyById(session.getId() + "");
            if(ent.getDealTime() == null)
                ent.setDealTime(date);
            mapper.updateById(ent);
        }
        GetSysInformReceiverViewOutput output = XCopyUtils.copyNewObject(ent , GetSysInformReceiverViewOutput.class);
        return XSingleResult.success(output);
    }

    @XText("更新接收方数据状态")
    @XApiPost
    public XServiceResult updateSysInformReceiverStatus(XContext context, UpdateSysInformReceiverListInput input){
        return XTransactionHelper.begin(context , ()->{
            if(CollectionUtils.isEmpty(input.getList()))
                return XServiceResult.error(200,"数据列表不能为空");
            PpsUserSession session = context.getSession(PpsUserSession.class);
            Date date = new Date();
            SysInformReceiverMapper mapper = context.getBean(SysInformReceiverMapper.class);
            for (String id : input.getList()){
                //先查询数据是否存在
                QueryWrapper<SysInformReceiverEnt> queryWrapper = new QueryWrapper<>();
                queryWrapper.lambda().eq(SysInformReceiverEnt::getId, id);
                SysInformReceiverEnt entity = mapper.selectOne(queryWrapper);
                if (entity == null) {
                    return XServiceResult.error(context, XError.NotFound);
                }
                entity.setStatus(input.getStatus());
                entity.setModifyTime(date);
                entity.setModifyByName(session.getUserName());
                entity.setModifyById(session.getId() + "");
                entity.setDealTime(date);
                mapper.updateById(entity);
            }
            return XServiceResult.OK;
        });
    }

    @XText("右下角通知数据")
    @XApiGet
    public XListResult<GetLowerRightCornerInformOutput> getLowerRightCornerInformList(XContext context, GetSysInformReceiverViewInput input){
        //1.取消息通知数据
        SysInformReceiverViewMapper mapper = context.getBean(SysInformReceiverViewMapper.class);
        PpsUserSession session = context.getSession(PpsUserSession.class);
        input.setUserId(session.getId());
        input.setStatus(0);
        SysInformReceiverView record = new SysInformReceiverView();
        XCopyUtils.copyObject(input, record);
        record.setInvalidTime(new Date());
        List<SysInformReceiverView> list = mapper.selectList(record);
        List<GetLowerRightCornerInformOutput> outputs = new ArrayList<>();
        list.stream().forEach(item->{
            GetLowerRightCornerInformOutput output = XCopyUtils.copyNewObject(item ,GetLowerRightCornerInformOutput.class);
            output.setType(0);
            outputs.add(output);
        });
        //2.取调度令通知数据
        DispOrderInforCloudService cloudService = context.getBean(DispOrderInforCloudService.class);
        GetDispOrderInfoCloudInput cloudInput = new GetDispOrderInfoCloudInput();
        cloudInput.setCompleteState(0);
        XListResult<GetDispOrderInfoCloudOutput> outputXListResult = cloudService.getReceivingList(context , cloudInput);
        outputXListResult.throwIfFail();
        List<GetDispOrderInfoCloudOutput> outputList = outputXListResult.getResult();
        outputList.stream().forEach(item->{
            GetLowerRightCornerInformOutput output = new GetLowerRightCornerInformOutput();
            output.setId(item.getId());
            output.setTitle(item.getName());
            output.setSendTime(item.getPostTime());
            output.setSendUserId(item.getSignerById());
            output.setSendUserName(item.getSignerByName());
            output.setSendOuId(item.getOuId());
            output.setSendOuName(item.getOuName());
            output.setType(1);
            outputs.add(output);
        });
        //3.取交接班通知数据
        DispDutyMatterYqdkzxInfCloudService yqdkzxCloudService = context.getBean(DispDutyMatterYqdkzxInfCloudService.class);
        XListResult<GetDispDutyMatterYqdkzxInfTypeDetaiCloudOutput> yqdkzxOutputXListResult = yqdkzxCloudService.getDispDutyMatterYqdkznInfList(context);
        yqdkzxOutputXListResult.throwIfFail();
        List<GetDispDutyMatterYqdkzxInfTypeDetaiCloudOutput> yqdkzxOutputList = yqdkzxOutputXListResult.getResult();
        yqdkzxOutputList.stream().forEach(item->{
            GetLowerRightCornerInformOutput output = new GetLowerRightCornerInformOutput();
            output.setId(item.getId());
            output.setTitle(item.getTheme());
            output.setSendTime(item.getModifyTime());
            output.setSendUserId(item.getSubmitterId());
            output.setSendUserName(item.getSubmitterName());
            output.setSendOuId(item.getOuId());
            output.setSendOuName(item.getOuName());
            output.setType(2);
            outputs.add(output);
        });
        return XListResult.success(outputs);
    }

    @XText("忽略当前用户消息通知")
    @XApiPost
    public XServiceResult ignorSysInformReceiver(XContext context, UpdateSysInformReceiverListInput input){
        return XTransactionHelper.begin(context , ()->{
            PpsUserSession session = context.getSession(PpsUserSession.class);
            Date date = new Date();
            //查询当前用户未查看的数据
            SysInformReceiverMapper mapper = context.getBean(SysInformReceiverMapper.class);
            QueryWrapper<SysInformReceiverEnt> queryWrapper = new QueryWrapper<>();
            queryWrapper.lambda().eq(SysInformReceiverEnt::getUserId, session.getId());
            queryWrapper.lambda().eq(SysInformReceiverEnt::getStatus, 0);
            List<SysInformReceiverEnt> list = mapper.selectList(queryWrapper);
            for (SysInformReceiverEnt entity : list){
                entity.setStatus(2);
                entity.setModifyTime(date);
                entity.setModifyByName(session.getUserName());
                entity.setModifyById(session.getId() + "");
                entity.setDealTime(date);
                mapper.updateById(entity);
            }
            return XServiceResult.OK;
        });
    }
}
