package pps.core.system.service;

import org.apache.commons.lang3.StringUtils;
import org.springframework.util.CollectionUtils;
import pps.cloud.primbpm.enums.ExtendAttributeOuLevelDic;
import pps.cloud.system.enums.OuLevelDic;
import pps.cloud.system.service.SysOrganizationCloudService;
import pps.cloud.system.service.SysRoleUserService;
import pps.cloud.system.service.data.*;
import pps.core.system.entity.SysAuthOuScopeUserInfo;
import pps.core.system.entity.SysAuthOuScopeView;
import pps.core.system.entity.SysUserRoleRelView;
import pps.core.system.mapper.SysAuthOuScopeViewMapper;
import pps.core.system.mapper.SysUserRoleRelViewMapper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.exception.XServiceException;
import xstartup.data.XListResult;
import xstartup.data.XSingleResult;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

@XText("角色用户管理")
@XService
public class SysRoleUserServiceImpl implements SysRoleUserService {

    @XText("查询用户列表")
    @Override
    public XListResult<QuerySysRoleUserOutput> querySysRoleUser(XContext context, QuerySysRoleUserInput input) {
        String ouId = null;
        String ouLevel = input.getOuLevel();
        switch (ExtendAttributeOuLevelDic.findByValue(ouLevel)){
            case PEER:
            case CURRENT_PEER:
            case APPOINT:
                ouId = input.getOuId();
                break;
            case SUPERIOR:
            case CURRENT_SUPERIOR:
                ouId = getParentOuInfo(context,input.getOuId()).getId();
                break;
            case SUPERIOR_PARENT:
            case CURRENT_SUPERIOR_PARENT:
                ouId = getParentOuInfo(context,input.getOuId()).getId();
                ouId = getParentOuInfo(context,ouId).getId();
                break;
            case PEER_OR_SUPERIOR:
            case CURRENT_PEER_OR_SUPERIOR:
                ouId = input.getOuId();
                ouId = ouId +","+ getParentOuInfo(context,input.getOuId()).getId();
                break;
            case NO:
                break;
            default:
                ouId = getOuIdByLevel(context,input.getOuId(),ExtendAttributeOuLevelDic.findByValue(ouLevel));
                break;
        }

        if(StringUtils.isBlank(ouId)){
            throw new XServiceException(100,"ou_level 配置错误");
        }
        SysOrganizationCloudService orgService = context.getBean(SysOrganizationCloudService.class);
        SysUserRoleRelViewMapper mapper = context.getBean(SysUserRoleRelViewMapper.class);
        List<String> ouIdList = Arrays.asList(ouId.split(","));
        List<QuerySysRoleUserOutput> retList = new ArrayList<>();
        for (String ouIdStr : ouIdList) {
            String ouLevelById = getOuLevelById(context, ouIdStr);
            SysUserRoleRelView record = new SysUserRoleRelView();
            record.setRoleId(input.getRoleId());
            record.setOuId(ouIdStr);
            record.setOuLevel(ouLevelById);
            List<SysUserRoleRelView> items = mapper.selectUserList(record);
            GetSysOrganizationViewInput orgInput = new GetSysOrganizationViewInput();
            orgInput.setId(ouIdStr);
            XSingleResult<GetSysOrganizationViewOutput> organization = orgService.getSysOrganizationById(context, orgInput);
            organization.throwIfFail();
            for (SysUserRoleRelView item : items) {
                QuerySysRoleUserOutput out = new QuerySysRoleUserOutput();
                if(StringUtils.isNotBlank(item.getIamLoginName())){
                    out.setId(item.getIamLoginName());
                }else{
                    out.setId(item.getLoginName());
                }
                out.setName(item.getUserName());
                out.setOuName(organization.getResult().getOuName());
                retList.add(out);
            }
            //获取范围授权用户
            if(input.getMenuId()!=null){
                SysAuthOuScopeView scopeView = new SysAuthOuScopeView();
                scopeView.setOuId(ouId);
                scopeView.setMenuId(input.getMenuId());
                SysAuthOuScopeViewMapper scopeViewMapper = context.getBean(SysAuthOuScopeViewMapper.class);
                List<SysAuthOuScopeUserInfo> userList = scopeViewMapper.selectScopeUserInfo(scopeView);
                if(!CollectionUtils.isEmpty(userList)){
                    for (SysAuthOuScopeUserInfo sysAuthOuScopeUserInfo : userList) {
                        QuerySysRoleUserOutput out = new QuerySysRoleUserOutput();
                        if(StringUtils.isNotBlank(sysAuthOuScopeUserInfo.getIamLoginName())){
                            out.setId(sysAuthOuScopeUserInfo.getIamLoginName());
                        }else{
                            out.setId(sysAuthOuScopeUserInfo.getLoginName());
                        }
                        out.setName(sysAuthOuScopeUserInfo.getUserName());
                        out.setOuName(organization.getResult().getOuName());
                        retList.add(out);
                    }
                }
            }
        }
        return XListResult.success(retList);
    }

    private String getOuIdByLevel(XContext context, String ouId, ExtendAttributeOuLevelDic ouLevelDic){
        if(ouId.contains(",")){
            ouId = ouId.split(",")[0];
        }
        String ouLevelById = getOuLevelById(context, ouId);
        if(ouLevelById.equals(ouLevelDic.getValue())){
            return ouId;
        }
        while (true){
            GetSysOrganizationViewOutput parentOuInfo = getParentOuInfo(context, ouId);
            if(parentOuInfo.getOuLevel().equals(ouLevelDic.getValue())){
                return parentOuInfo.getId();
            }else{
                ouId = parentOuInfo.getId();
            }
        }
    }

    private GetSysOrganizationViewOutput getParentOuInfo(XContext context, String ouId){
        GetSysOrganizationViewInput viewInput = new GetSysOrganizationViewInput();
        viewInput.setId(ouId);
        XSingleResult<GetSysOrganizationViewOutput> parentOrg = context.getBean(SysOrganizationCloudService.class).getOrganizationBySubOuId(context, viewInput);
        parentOrg.throwIfFail();
        return parentOrg.getResult();
    }

    private String getOuLevelById(XContext context,String ouId){
        if(ouId.contains(",")){
            ouId = ouId.split(",")[0];
        }
        GetSysOrganizationViewInput viewInput = new GetSysOrganizationViewInput();
        viewInput.setId(ouId);
        XSingleResult<GetSysOrganizationViewOutput> sysOrganizationById = context.getBean(SysOrganizationCloudService.class).getSysOrganizationById(context, viewInput);
        sysOrganizationById.throwIfFail();
        return sysOrganizationById.getResult().getOuLevel();
    }

    @XText("查询ou_level, 非部门")
    public XSingleResult<GetOuLevelOutput> queryOuLevel(XContext context, GetOuLevelInput input) {
        String outOuLevel = null;
        String ouId = null;
        String ouLevel = input.getOuLevel();
        switch (ExtendAttributeOuLevelDic.findByValue(ouLevel)){
            case PEER:
            case CURRENT_PEER:
            case APPOINT:
                ouId = input.getOuId();
                break;
            case SUPERIOR:
            case CURRENT_SUPERIOR:
                ouId = getParentOuInfo(context,input.getOuId()).getId();
                break;
            case SUPERIOR_PARENT:
            case CURRENT_SUPERIOR_PARENT:
                ouId = getParentOuInfo(context,input.getOuId()).getId();
                ouId = getParentOuInfo(context,ouId).getId();
                break;
            case PEER_OR_SUPERIOR:
            case CURRENT_PEER_OR_SUPERIOR:
                ouId = input.getOuId();
                break;
            case NO:
                break;
            default:
                outOuLevel = ouLevel;
                break;
        }

        if(ouId!=null){
            String ouLevelById = getOuLevelById(context, ouId);
            if(ouLevelById.equals(OuLevelDic.OuLevelCS.getKey()) || ouLevelById.equals(OuLevelDic.OuLevelKS.getKey())){
                outOuLevel = getParentOuInfo(context,ouId).getOuLevel();
            }else {
                outOuLevel = ouLevelById;
            }
        }
        GetOuLevelOutput output = new GetOuLevelOutput();
        output.setOuLevel(outOuLevel);
        return XSingleResult.success(output);
    }



}
