package pps.core.task.job;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pps.cloud.scadapi.service.ScadaPiService;
import pps.cloud.scadapi.service.data.GetTagsInput;
import pps.cloud.scadapi.service.data.SyncScadaDataParamsInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;
import java.util.GregorianCalendar;


@XText("SCADAPI Tag点值 数据同步 采集频率为 1 小时")
@XService
public class ScadaPiDataJob implements XJob {
    private static Logger logger = LoggerFactory.getLogger(ScadaPiJob.class);
    @Override
    @XCronTrigger(value = "0 * * * * ?")
    public XServiceResult execute(XContext context) {

        /***
         * 0.小时采集服务 采集频率为 1
         * 1.向前采集 or 向后采集 【本服务向前采集】
         * 2.采集时间 为当前时间 例如：【07:49;07:55;07:57;08:00,08:01;08:03;08:05;08:10】
         * 3.通过偏移量【分钟级别】 取 0 - 59 ，30 以前 的数据 存储到 当前时间点 30以后的数据存储到 下一个时间点
         * 4.采集频率 【1】
         * 5.根据 [采集方向、偏移量、采集频率] 这 3 个 字段类型 卡 scada_gather_conf 的数据
         */

        Date currentTime = new Date();
        int currentMin = currentTime.getMinutes();
        String gather_offsetStr = String.valueOf(currentMin);

        SyncScadaDataParamsInput input = new SyncScadaDataParamsInput();

        input.setCollectInterval(1);//采集频率为 1 小时
        input.setGatherOffset(gather_offsetStr);//gatherOffset
        input.setCurrentTime(currentTime);//currentTime
        input.setSyncType(0); //syncType
        input.setMoveTowards("1"); //向前向后（1-向前；2-向后）

        SimpleDateFormat formatzxsj = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

        input.setTaskName("普通值 频率 1 执行时间为："+formatzxsj.format(currentTime) );

        ScadaPiService service = context.getBean(ScadaPiService.class);
        service.SyncScadaDataParams(context, input);

        return XServiceResult.OK;
    }
}
