package pps.core.task.job;

import pps.cloud.scadapi.service.ScadaPiService;
import pps.cloud.scadapi.service.data.SyncScadaDataParamsInput;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Date;

@XText("SCADAPI 交接量采集 采集类型为交接量的点 取每天 7:31以后的第一个值")
@XService
public class ScadaPiDataOilAmountJob implements XJob {

    /***
     * 计量数据采集 服务  每天 8：03 开始采集【偏移量配置的 3】 每隔 5 分钟采集一次 执行 半个小时 到08：30
     * @param xContext
     * @return
     */
    @Override
    @XCronTrigger(value = "0 0,5,10,15,20 8,10,11,15,16,17,18,19,21,22 * * ?")
    public XServiceResult execute(XContext xContext) {

        /***
         * 0.【0 0/3 8 * * ?】 表示 每天 8 点 每3分钟执行一次
         * 1.向前采集 or 向后采集 【本服务向后采集】
         * 2.采集时间 为当前时间 例如：【07:49;07:55;07:57;08:00,08:01;08:03;08:05;08:10】
         * 3.通过偏移量【分钟级别 绝对值小于30】 去卡时间 偏移量配到到 时间表达式 中【查询接口的时候 获取 当前时间的分钟数据 大于30的偏移量 = 分钟数 - 60 小于30的偏移量 = 分钟数】
         * 4.采集类型 【交接量】【普通值】 【本服务取交接量】
         * 5.采集频率 【24】
         * 5.根据 [采集方向、偏移量、采集类型、采集频率] 这 4 个 字段类型 卡 scada_gather_conf 的数据
         */

        Date currentTime = new Date();
        int currentMin = currentTime.getMinutes();
        int gather_offset = currentMin;
        if(currentMin>30){
            gather_offset = currentMin - 60;
        }

        SyncScadaDataParamsInput input = new SyncScadaDataParamsInput();

        SimpleDateFormat format0750 = new SimpleDateFormat("yyyy-MM-dd 07:50:00");//计量数据取 07:50 之后的第一个数据

        String date0750 = format0750.format(currentTime);

        SimpleDateFormat formatMMSS = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");//计量数据取 07:50 之后的第一个数据

        try {
            input.setCurrentTime(formatMMSS.parse(date0750));//计量数据取 07:50 之后的第一个数据
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }

        input.setSyncType(0); //正常取数据 or 补采集数据 0：正常取数据 1：补采数据
        input.setMoveTowards("2");// 向前向后（1-向前；2-向后）
        input.setCollectInterval(24);//交接量采集 频率都是 24 表示 一天就一个值 存储到 早八点
        input.setGatherOffset("3");//String.valueOf(gather_offset); //交接量的便宜量都为 3 分钟 及 8：03 开始执行 3是为了查询scadaconf 配置数据 不是3的话 查询不到数据
        input.setGatherType("2");//采集类型（1-常规；2-自定义 ,2：为交接量） //交接量的类型为 2

        input.setTaskName("计量数据 频率 24 执行时间为："+formatMMSS.format(currentTime));

        ScadaPiService service = xContext.getBean(ScadaPiService.class);
        service.SyncScadaDataParams(xContext, input);

        return XServiceResult.OK;
    }
}








