package pps.core.task.job;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import pps.cloud.base.service.BaseSysConfigCloudService;
import pps.cloud.base.service.data.ConfigValueInput;
import pps.cloud.transfer.service.TransferService;
import pps.cloud.transfer.service.data.SyncOldPpsImsTaskInput;
import pps.core.task.config.TaskConfig;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.util.XDateUtils;
import xstartup.data.XServiceResult;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

import java.util.Date;

import static xstartup.service.job.annotation.XCronTrigger.PRE_30S;

@XText("作业计划ims同步任务")
@XService
public class TaskPlanImsJob implements XJob {

    private static Logger logger = LoggerFactory.getLogger(TaskPlanImsJob.class);

    @XCronTrigger(value = "0/35 * * * * ?")
    @Override
    public XServiceResult execute(XContext context) {
        String lastDate = new TaskConfig().taskPlanImsDate.findValue(context);
        if(StringUtils.isBlank(lastDate)){
            context.getLogger().error("作业计划ims同步时间未配置");
            return XServiceResult.OK;
        }
        Date now = new Date();
        TransferService transferService = context.getBean(TransferService.class);
        SyncOldPpsImsTaskInput syncOldPpsImsTaskInput = new SyncOldPpsImsTaskInput();
        syncOldPpsImsTaskInput.setLastDate(XDateUtils.addMunites(XDateUtils.parse(lastDate),-10)); //由于系统时钟不同步，故加大10分钟范围
        syncOldPpsImsTaskInput.setNowDate(now);
        XServiceResult serviceResult = transferService.syncOldPpsImsTask(context, syncOldPpsImsTaskInput);
        serviceResult.throwIfFail();

        //更新最后获取时间
        BaseSysConfigCloudService bean = context.getBean(BaseSysConfigCloudService.class);
        ConfigValueInput configValueInput = new ConfigValueInput();
        configValueInput.setConfigKey("pps.core.task.task-config.taskPlanImsDate");
        configValueInput.setConfigValue(XDateUtils.getString(now));
        bean.setConfigValue(context,configValueInput);
        return XServiceResult.OK;
    }
}
