package pps.core.task.job;

import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.util.CollectionUtils;
import pps.cloud.dispatch.service.DispTaskPlanJobService;
import pps.cloud.dispatch.service.data.TaskPlanTransferModel;
import pps.cloud.dispatch.service.data.TaskPlanTransferNewInput;
import pps.cloud.dispatch.service.data.TaskPlanTransferNewOutput;
import pps.cloud.transfer.service.TransferService;
import pps.cloud.transfer.service.data.QueryOldPpsImsTaskInput;
import pps.cloud.transfer.service.data.QueryOldPpsImsTaskOutput;
import pps.cloud.transfer.service.data.UpdateOldPpsImsTaskInput;
import pps.cloud.transfer.service.data.UpdateOldPpsModel;
import pps.core.common.utils.UUIDHelper;
import xstartup.annotation.XService;
import xstartup.annotation.XText;
import xstartup.base.XContext;
import xstartup.base.XStartup;
import xstartup.data.XListResult;
import xstartup.data.XServiceResult;
import xstartup.data.XSingleResult;
import xstartup.helper.XLockHelper;
import xstartup.service.job.XJob;
import xstartup.service.job.annotation.XCronTrigger;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import static xstartup.service.job.annotation.XCronTrigger.PRE_30S;

@XText("作业计划转换成新作业计划任务")
@XService
public class TaskPlanTransferJob implements XJob {

    private static Logger logger = LoggerFactory.getLogger(TaskPlanTransferJob.class);

    @XCronTrigger(value = "0/35 * * * * ?")
    @Override
    public XServiceResult execute(XContext context) {
        String lockkey = "old_task_job" + XStartup.getApplication().getEnv().toString();
        XLockHelper.lock(context, "old_task_job", lockkey, 60, () -> {
            //获取老pps作业计划
            TransferService transferService = context.getBean(TransferService.class);
            QueryOldPpsImsTaskInput oldTaskInput = new QueryOldPpsImsTaskInput();
            XListResult<QueryOldPpsImsTaskOutput> oldTaskListResult = transferService.queryOldPpsImsTask(context, oldTaskInput);
            oldTaskListResult.throwIfFail();
            List<QueryOldPpsImsTaskOutput> oldTaskList = oldTaskListResult.getResult();
            if(CollectionUtils.isEmpty(oldTaskList)){
                return;
            }
            //转换作业计划
            List<TaskPlanTransferModel> newTaskList = new ArrayList<>();
            Date now = new Date();
            for (QueryOldPpsImsTaskOutput oldTask : oldTaskList) {
                TaskPlanTransferModel model = new TaskPlanTransferModel();
                model.setTransferId(oldTask.getId());
                model.setId(UUIDHelper.newUUID());
                model.setTaskCode(oldTask.getJobPlanCodeNew());
                model.setTaskCodeIms(oldTask.getJobPlanCodeIMS());
                model.setTaskName(oldTask.getJobPlanName());
                model.setPipelineId(oldTask.getPipelineID().toUpperCase());
                model.setPipelineName(oldTask.getPipelineName());
                model.setTaskLevel(oldTask.getJobLevel()); //字典转换
                model.setOuId(oldTask.getCompanyOUID().toUpperCase());
                model.setOuName(oldTask.getCompanyOuName());
                model.setBranchOuName(oldTask.getOuName());
                model.setTaskOuId(oldTask.getCompanyOUID().toUpperCase());
                model.setTaskOuName(oldTask.getCompanyOuName());
                model.setTaskOuType("ou"); //作业单位类型（ou:组织机构，station：场站）
                model.setUnitOuId(oldTask.getoUID().toUpperCase());
                model.setPrincipal(oldTask.getJobPlanResponsible());
                model.setContactInfo(oldTask.getContactWay());
                model.setTaskScope(oldTask.getJobScopeType()); //字典转换
                model.setTaskType(oldTask.getJobPlanType()); //字典转换
                model.setTaskNature(oldTask.getJobPropertyType()); //字典转换
                model.setMajor(oldTask.getProfessionalType()); //字典转换
                model.setExpectStartDate(oldTask.getEstimateStartTime());
                model.setExpectEndDate(oldTask.getEstimateEndTime());
                if (StringUtils.isNotBlank(oldTask.getBeginStationID())) {
                    model.setStartStationId(oldTask.getBeginStationID().toUpperCase());
                    model.setStartStationName(oldTask.getStartStationName());
                }
                if (StringUtils.isNotBlank(oldTask.getEndStationID())) {
                    model.setEndStationId(oldTask.getEndStationID().toUpperCase());
                    model.setEndStationName(oldTask.getEndStationName());
                }
                if (StringUtils.isNotBlank(oldTask.getStationID())) {
                    model.setStartStationId(oldTask.getStationID().toUpperCase());
                    model.setStartStationName(oldTask.getStationName());
                }
                if (StringUtils.isNotBlank(oldTask.getValveboxStationID())) {
                    model.setStartStationId(oldTask.getValveboxStationID().toUpperCase());
                    model.setStartStationName(oldTask.getValveName());
                }
                //操作影响的处理 --start
                model.setEffect(oldTask.getEffect());
                model.setIsAffectAbility(oldTask.getTransferSupply());
                model.setIsAffectCommunication(oldTask.getEffectCommunicate());
                model.setIsAffectRun(oldTask.getEffectUnitOperation());
                model.setIsAffectSupplyGas(oldTask.getTransferSupply());
                Integer naturalUpload = oldTask.getNaturalUpload();
                if(naturalUpload!=null){
                    if(naturalUpload==0){
                        model.setIsAffectUpload(1);
                    }else{
                        model.setIsAffectUpload(0);
                    }
                }
                model.setIsAffectVenting(oldTask.getVenting());
                model.setIsProduceAlarm(oldTask.getGenerateAlarm());

                model.setAffectAbilityText(oldTask.getTransferSupplyContent());
                model.setAffectCommunicationText(oldTask.getEffectCommunicateContent());
                model.setAffectRunText(oldTask.getEffectUnitOperationContent());
                model.setAffectSupplyGasText(oldTask.getTransferSupplyContent());
                model.setAffectUploadText(oldTask.getNaturalUploadContent());
                model.setAffectVentingText(oldTask.getVentingContent());
                model.setProduceAlarmText(oldTask.getGenerateAlarmContent());
                //操作影响的处理 --end
                model.setTaskLevel(oldTask.getJobPlanLevelIMS());
                Integer pipelineLevelIMS = oldTask.getPipelineLevel();
                if(pipelineLevelIMS==null){
                    pipelineLevelIMS = 1; //默认一级
                }
                model.setPipelineLevel(pipelineLevelIMS);

                model.setTaskContent(oldTask.getJobContent());
                model.setMainWorkFlow(oldTask.getOperationFlow());
                model.setRemark(oldTask.getRemark());
                model.setStatus(1); // 暂存
                //周期计算
                //model.setCycleValue(getCycleValue(model.getTaskType(),model.getExpectStartDate()));
                model.setCreateById("11111111-1111-1111-1111-111111111111");
                model.setCreateByName(oldTask.getCreateByName());
                model.setCreateTime(now);
                model.setCreateByOuId(oldTask.getoUID().toUpperCase()); //待校准
                model.setMediaType(String.valueOf(oldTask.getMedium()));//字典转换
                newTaskList.add(model);
            }
            //插入新pps作业计划
            DispTaskPlanJobService taskPlanJobService = context.getBean(DispTaskPlanJobService.class);
            TaskPlanTransferNewInput newInput = new TaskPlanTransferNewInput();
            newInput.setList(newTaskList);
            XSingleResult<TaskPlanTransferNewOutput> serviceResult = taskPlanJobService.insertImsTaskPlan(context, newInput);
            serviceResult.throwIfFail();

            //更新老pps作业计划转换状态
            if(!CollectionUtils.isEmpty(serviceResult.getResult().getList())) {
                UpdateOldPpsImsTaskInput updateInput = new UpdateOldPpsImsTaskInput();
                List<UpdateOldPpsModel> updateList = new ArrayList<>();
                for (TaskPlanTransferModel model : serviceResult.getResult().getList()) {
                    UpdateOldPpsModel oldPpsModel = new UpdateOldPpsModel();
                    oldPpsModel.setId(model.getTransferId());
                    oldPpsModel.setErrorMsg(model.getErrorMsg());
                    if(model.getStatus()==null){
                        oldPpsModel.setStatus(1);
                    }else{
                        oldPpsModel.setStatus(model.getStatus());
                    }
                    updateList.add(oldPpsModel);
                }
                updateInput.setList(updateList);
                XServiceResult serviceResult1 = transferService.updateOldPpsImsTask(context, updateInput);
                serviceResult1.throwIfFail();
            }
        });
        return XServiceResult.OK;
    }
}
